==> libtorrent-0.7.6.tar.gz: libtorrent-0.7.6/src/net/socket_manager.cc <==
// libTorrent - BitTorrent library
// Copyright (C) 2005, Jari Sundell
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// In addition, as a special exception, the copyright holders give
// permission to link the code of portions of this program with the
// OpenSSL library under certain conditions as described in each
// individual source file, and distribute linked combinations
// including the two.
//
// You must obey the GNU General Public License in all respects for
// all of the code used other than OpenSSL.  If you modify file(s)
// with this exception, you may extend this exception to your version
// of the file(s), but you are not obligated to do so.  If you do not
// wish to do so, delete this exception statement from your version.
// If you delete this exception statement from all source files in the
// program, then also delete it here.
//
// Contact:  Jari Sundell <jaris@ifi.uio.no>
//
//           Skomakerveien 33
//           3185 Skoppum, NORWAY

#include "config.h"

#include "torrent/exceptions.h"

#include "socket_address.h"
#include "socket_manager.h"

namespace torrent {

SocketFd
SocketManager::open(const SocketAddress& sa, const SocketAddress& b) {
  SocketFd fd;

  if (m_size >= m_max || !fd.open_stream())
    return SocketFd();

  if (!fd.set_nonblock() ||
      (!b.is_address_any() && !fd.bind(b)) ||
      !fd.connect(sa)) {
    fd.close();
    return SocketFd();
  }

  m_size++;
  return fd;
}

SocketFd
SocketManager::received(SocketFd fd, const SocketAddress& sa) {
  if (!fd.is_valid())
    throw internal_error("SocketManager::received(...) received an invalid file descriptor");

  if (m_size >= m_max ||
      !fd.set_nonblock()) {
    fd.close();
    return SocketFd();
  }

  m_size++;
  return fd;
}

void
SocketManager::close(SocketFd fd) {
  if (!fd.is_valid())
    throw internal_error("SocketManager::close(...) received an invalid file descriptor");

  fd.close();
  m_size--;
}

}

==> libtorrent-0.8.3.tar.gz: libtorrent-0.8.3/src/net/socket_manager.cc <==
// libTorrent - BitTorrent library
// Copyright (C) 2005-2006, Jari Sundell
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// In addition, as a special exception, the copyright holders give
// permission to link the code of portions of this program with the
// OpenSSL library under certain conditions as described in each
// individual source file, and distribute linked combinations
// including the two.
//
// You must obey the GNU General Public License in all respects for
// all of the code used other than OpenSSL.  If you modify file(s)
// with this exception, you may extend this exception to your version
// of the file(s), but you are not obligated to do so.  If you do not
// wish to do so, delete this exception statement from your version.
// If you delete this exception statement from all source files in the
// program, then also delete it here.
//
// Contact:  Jari Sundell <jaris@ifi.uio.no>
//
//           Skomakerveien 33
//           3185 Skoppum, NORWAY

#include "config.h"

#include "torrent/exceptions.h"

#include "socket_address.h"
#include "socket_manager.h"

namespace torrent {

SocketFd
SocketManager::open(const SocketAddress& sa, const SocketAddress& b) {
  SocketFd fd;

  if (m_size >= m_max || !fd.open_stream())
    return SocketFd();

  if (!fd.set_nonblock() ||
      (!b.is_address_any() && !fd.bind(b)) ||
      !fd.connect(sa)) {
    fd.close();
    return SocketFd();
  }

  m_size++;
  return fd;
}

SocketFd
SocketManager::received(SocketFd fd, const SocketAddress& sa) {
  if (!fd.is_valid())
    throw internal_error("SocketManager::received(...) received an invalid file descriptor");

  if (m_size >= m_max || !fd.set_nonblock()) {
    fd.close();
    return SocketFd();
  }

  m_size++;
  return fd;
}

void
SocketManager::close(SocketFd fd) {
  if (!fd.is_valid())
    throw internal_error("SocketManager::close(...) received an invalid file descriptor");

  fd.close();
  m_size--;
}

}
